/* kgx-process-dummy.c
 *
 * Copyright 2019 Zander Brown
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "kgx-process.h"

/*
 * Provided to simplify gtop-free builds, does nothing and never will
 * 
 * If the gtop-free build is working it should never even run
 */

void
kgx_process_unref (KgxProcess *self)
{
  g_critical ("%s shouldn't be reached", G_STRLOC);
}

int
kgx_pid_cmp (gconstpointer a, gconstpointer b, gpointer data)
{
  g_critical ("%s shouldn't be reached", G_STRLOC);
  
  return 0;
}
